@@ -5,13 +5,13 @@ import os
 import sys
 from typing import Any
 
-from aiohttp import ClientSession
+from aiohttp import ClientResponse, ClientSession
 from yarl import URL
 
 from .components import Callback, HttpComponent, MetablockResponseError
-from .extensions import Extensions, Plugins
-from .orgs import Orgs
-from .spaces import Blocks, Domains, Space, Spaces
+from .extensions import Extension, Extensions, Plugin, Plugins
+from .orgs import Org, Orgs
+from .spaces import Block, Blocks, Domains, Space, Spaces
 from .user import User
 
 DEFAULT_USER_AGENT = f"Python/{'.'.join(map(str, sys.version_info[:2]))} metablock"
@@ -41,13 +41,13 @@ class Metablock(HttpComponent):
             "user-agent": user_agent,
             "accept": "application/json",
         }
-        self.spaces: Spaces = Spaces(self)
+        self.orgs: Orgs = Orgs(self, Org)
+        self.spaces: Spaces = Spaces(self, Space)
+        self.blocks: Blocks = Blocks(self, Block, "services")
+        self.plugins: Plugins = Plugins(self, Plugin)
+        self.extensions: Extensions = Extensions(self, Extension)
         self.domains = Domains(self)
-        self.orgs: Orgs = Orgs(self)
-        self.blocks: Blocks = Blocks(self, "services")
         self.services = self.blocks
-        self.plugins: Plugins = Plugins(self)
-        self.extensions: Extensions = Extensions(self)
 
     def __repr__(self) -> str:
         return self.url
@@ -69,9 +69,29 @@ class Metablock(HttpComponent):
         await self.close()
 
     async def spec(self) -> dict:
-        return await self.execute(f"{self.url}/spec")
+        return await self.request(f"{self.url}/spec")
 
-    async def execute(
+    async def get(self, url: str | URL, **kwargs: Any) -> Any:
+        kwargs["method"] = "GET"
+        return await self.request(url, **kwargs)
+
+    async def patch(self, url: str | URL, **kwargs: Any) -> Any:
+        kwargs["method"] = "PATCH"
+        return await self.request(url, **kwargs)
+
+    async def post(self, url: str | URL, **kwargs: Any) -> Any:
+        kwargs["method"] = "POST"
+        return await self.request(url, **kwargs)
+
+    async def put(self, url: str | URL, **kwargs: Any) -> Any:
+        kwargs["method"] = "PUT"
+        return await self.request(url, **kwargs)
+
+    async def delete(self, url: str | URL, **kwargs: Any) -> Any:
+        kwargs["method"] = "DELETE"
+        return await self.request(url, **kwargs)
+
+    async def request(
         self,
         url: str | URL,
         method: str = "",
@@ -88,6 +108,10 @@ class Metablock(HttpComponent):
         response = await self.session.request(method, url, headers=headers_, **kw)
         if callback:
             return await callback(response)
+        else:
+            return await self.handle_response(response, wrap=wrap)
+
+    async def handle_response(self, response: ClientResponse, wrap: Any = None) -> Any:
         if response.status == 204:
             return True
         if response.status >= 400:
@@ -100,19 +124,20 @@ class Metablock(HttpComponent):
         data = await response.json()
         return wrap(data) if wrap else data
 
-    async def get_user(self, callback: Callback | None = None) -> User:
-        return await self.get(f"{self.url}/user", callback=callback, wrap=self._user)
+    async def get_user(self, **kw: Any) -> User:
+        kw.setdefault("wrap", self._user)
+        return await self.get(f"{self.url}/user", **kw)
 
-    async def get_space(self, callback: Callback | None = None) -> Space:
-        return await self.get(f"{self.url}/space", callback=callback, wrap=self._space)
+    async def get_space(self, **kw: Any) -> Space:
+        kw.setdefault("wrap", self._space)
+        return await self.get(f"{self.url}/space", **kw)
 
-    async def update_user(self, callback: Callback | None = None, **data: Any) -> User:
-        return await self.patch(
-            f"{self.url}/user", json=data, callback=callback, wrap=self._user
-        )
+    async def update_user(self, **kw: Any) -> User:
+        kw.setdefault("wrap", self._user)
+        return await self.patch(f"{self.url}/user", **kw)
 
-    async def delete_user(self, callback: Callback | None = None) -> None:
-        return await self.delete(f"{self.url}/user", callback=callback)
+    async def delete_user(self, **kw: Any) -> None:
+        return await self.delete(f"{self.url}/user", **kw)
 
     def get_default_headers(self) -> dict[str, str]:
         headers = self.default_headers.copy()
