@@ -1,11 +1,20 @@
 from __future__ import annotations
 
 import json
-from abc import ABC, abstractmethod
-from typing import TYPE_CHECKING, Any, AsyncIterator, Awaitable, Callable, cast
+from abc import ABC, abstractproperty
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    AsyncIterator,
+    Awaitable,
+    Callable,
+    Generic,
+    Mapping,
+    TypeVar,
+    cast,
+)
 
 from aiohttp import ClientResponse
-from yarl import URL
 
 from .utils import as_dict, as_params
 
@@ -37,32 +46,43 @@ class MetablockResponseError(MetablockError):
 
 
 class HttpComponent(ABC):
-    @abstractmethod
-    def execute(self, url: str, *, method: str = "", **kwargs: Any) -> Any:
-        """Execute Http request"""
+    @abstractproperty
+    def cli(self) -> Metablock:  # pragma: no cover
+        ...
 
-    async def get(self, url: str, **kwargs: Any) -> Any:
-        kwargs["method"] = "GET"
-        return await self.execute(url, **kwargs)
+    @abstractproperty
+    def url(self) -> str:  # pragma: no cover
+        ...
 
-    async def patch(self, url: str, **kwargs: Any) -> Any:
-        kwargs["method"] = "PATCH"
-        return await self.execute(url, **kwargs)
+    def __repr__(self) -> str:
+        return self.url
 
-    async def post(self, url: str, **kwargs: Any) -> Any:
-        kwargs["method"] = "POST"
-        return await self.execute(url, **kwargs)
+    def __str__(self) -> str:
+        return self.__repr__()
 
-    async def put(self, url: str, **kwargs: Any) -> Any:
-        kwargs["method"] = "PUT"
-        return await self.execute(url, **kwargs)
 
-    async def delete(self, url: str, **kwargs: Any) -> Any:
-        kwargs["method"] = "DELETE"
-        return await self.execute(url, **kwargs)
+class Component(HttpComponent):
+    def __init__(
+        self, root: Metablock | CrudComponent | MetablockEntity, name: str = ""
+    ) -> None:
+        self.root = root
+        self.name = name or self.__class__.__name__.lower()
 
-    def wrap(self, data: Any) -> Any:
-        return data
+    @property
+    def cli(self) -> Metablock:
+        return self.root.cli
+
+    @property
+    def url(self) -> str:
+        return f"{self.cli.url}/{self.name}"
+
+    @property
+    def parent_url(self) -> str:
+        return f"{self.root.url}/{self.name}"
+
+    @property
+    def is_entity(self) -> bool:
+        return isinstance(self.root, MetablockEntity)
 
 
 class MetablockEntity(HttpComponent):
@@ -71,7 +91,9 @@ class MetablockEntity(HttpComponent):
     __slots__ = ("root", "data")
 
     def __init__(
-        self, root: Metablock | CrudComponent | MetablockEntity, data: dict
+        self,
+        root: Metablock | CrudComponent | MetablockEntity,
+        data: dict,
     ) -> None:
         self.root = root
         self.data = data
@@ -79,9 +101,6 @@ class MetablockEntity(HttpComponent):
     def __repr__(self) -> str:
         return repr(self.data)
 
-    def __str__(self) -> str:
-        return self.__repr__()
-
     def __getitem__(self, item: str) -> Any:
         return self.data[item]
 
@@ -110,129 +129,80 @@ class MetablockEntity(HttpComponent):
     def url(self) -> str:
         return "%s/%s" % (self.root.url, self.id)
 
-    async def execute(self, url: str | URL, **params: Any) -> Any:
-        return await self.cli.execute(url, **params)
-
     def nice(self) -> str:
         return json.dumps(self.data, indent=4)
 
 
-class Component:
-    __slots__ = ("root", "name")
+E = TypeVar("E", bound="MetablockEntity")
 
+
+class CrudComponent(Component, Generic[E]):
     def __init__(
-        self, root: Metablock | CrudComponent | MetablockEntity, name: str = ""
+        self,
+        root: Metablock | CrudComponent | MetablockEntity,
+        factory: type[E],
+        name: str = "",
     ) -> None:
-        self.root = root
-        self.name = name or self.__class__.__name__.lower()
+        super().__init__(root, name)
+        self.factory = factory
 
-    def __repr__(self) -> str:
-        return self.url
-
-    def __str__(self) -> str:
-        return self.__repr__()
-
-    @property
-    def cli(self) -> Metablock:
-        return self.root.cli
-
-    @property
-    def url(self) -> str:
-        return f"{self.cli.url}/{self.name}"
-
-
-class CrudComponent(HttpComponent):
-    Entity = MetablockEntity
-    __slots__ = ("root", "name")
-
-    def __init__(
-        self, root: Metablock | CrudComponent | MetablockEntity, name: str = ""
-    ) -> None:
-        self.root = root
-        self.name = name or self.__class__.__name__.lower()
-
-    def __repr__(self) -> str:
-        return self.url
-
-    def __str__(self) -> str:
-        return self.__repr__()
-
-    @property
-    def cli(self) -> Metablock:
-        return self.root.cli
-
-    @property
-    def url(self) -> str:
-        return f"{self.cli.url}/{self.name}"
-
-    @property
-    def parent_url(self) -> str:
-        return f"{self.root.url}/{self.name}"
-
-    @property
-    def is_entity(self) -> bool:
-        return isinstance(self.root, MetablockEntity)
-
-    async def execute(self, url: str | URL, **kwargs: Any) -> Any:
-        return await self.root.execute(url, **kwargs)
-
-    async def paginate(self, **params: Any) -> AsyncIterator[MetablockEntity]:
-        url = self.list_create_url()
-        next_ = url
-        url_params = as_params(**params)
+    async def paginate(self, **params: Any) -> AsyncIterator[E]:
+        next_ = self.list_create_url()
+        url_params: Any = as_params(**params)
         while next_:
-            if not next_.startswith(url):
-                next_ = f'{url}?{next_.split("?")[1]}'
-            data = await self.execute(next_, params=url_params)
-            next_ = data.get("next")
-            for d in data["data"]:
-                yield self.wrap(d)
+            next_, data = await self.cli.request(
+                next_, params=url_params, callback=self._paginated
+            )
+            url_params = None
+            for d in data:
+                yield self.entity(d)
 
-    async def get_list(self, **params: Any) -> list[MetablockEntity]:
+    async def get_list(self, **kwargs: Any) -> list[E]:
         url = self.list_create_url()
+        kwargs.setdefault("wrap", self.entity_list)
         return cast(
-            list[MetablockEntity],
-            await self.execute(url, params=as_params(**params), wrap=self.wrap_list),
+            list[E],
+            await self.cli.request(url, **kwargs),
         )
 
-    async def get_full_list(self, **params: Any) -> list[MetablockEntity]:
-        return [d async for d in self.paginate(**params)]
+    async def get_full_list(self, **kwargs: Any) -> list[E]:
+        return [d async for d in self.paginate(**kwargs)]
 
-    async def get(self, id_: str, **kwargs: Any) -> MetablockEntity:  # type: ignore
+    async def get(self, id_: str, **kwargs: Any) -> E:
         url = f"{self.url}/{id_}"
-        kwargs.update(wrap=self.wrap)
-        return cast(MetablockEntity, await self.execute(url, **kwargs))
+        kwargs.setdefault("wrap", self.entity)
+        return cast(E, await self.cli.get(url, **kwargs))
 
     async def has(self, id_: str, **kwargs: Any) -> bool:
         url = f"{self.url}/{id_}"
-        kwargs.update(wrap=self.head)
-        return cast(bool, await self.get(url, callback=self.head))
+        return cast(bool, await self.cli.get(url, callback=self._head))
 
-    async def create(
-        self, callback: Callback | None = None, **params: Any
-    ) -> MetablockEntity:
+    async def create(self, callback: Callback | None = None, **params: Any) -> E:
         url = self.list_create_url()
         return cast(
-            MetablockEntity,
-            await self.post(url, json=params, callback=callback, wrap=self.wrap),
+            E,
+            await self.cli.post(url, json=params, callback=callback, wrap=self.entity),
         )
 
     async def update(
         self, id_name: str, callback: Callback | None = None, **params: Any
-    ) -> MetablockEntity:
+    ) -> E:
         return cast(
-            MetablockEntity,
-            await self.patch(
-                self.update_url(id_name), json=params, callback=callback, wrap=self.wrap
+            E,
+            await self.cli.patch(
+                self.update_url(id_name),
+                json=params,
+                callback=callback,
+                wrap=self.entity,
             ),
         )
 
-    async def upsert(
-        self, callback: Callback | None = None, **params: Any
-    ) -> MetablockEntity:
+    async def upsert(self, callback: Callback | None = None, **params: Any) -> E:
         return cast(
-            MetablockEntity,
-            await self.put(self.url, json=params, callback=callback, wrap=self.wrap),
+            E,
+            await self.cli.put(
+                self.url, json=params, callback=callback, wrap=self.entity
+            ),
         )
 
     async def delete(  # type: ignore
@@ -249,22 +219,11 @@ class CrudComponent(HttpComponent):
             n += 1
         return n
 
-    async def head(self, response: ClientResponse) -> bool:
-        if response.status == 404:
-            return False
-        elif response.status == 200:
-            return True
-        else:  # pragma: no cover
-            raise MetablockResponseError(response)
+    def entity(self, data: dict) -> E:
+        return self.factory(self, data)
 
-    def wrap(self, data: dict) -> MetablockEntity:
-        return self.Entity(self, data)
-
-    def entity(self, **data: Any) -> MetablockEntity:
-        return self.Entity(self, data)
-
-    def wrap_list(self, data: list[dict]) -> list[MetablockEntity]:
-        return [self.wrap(d) for d in data]
+    def entity_list(self, data: list[dict]) -> list[E]:
+        return [self.entity(d) for d in data]
 
     def list_create_url(self) -> str:
         return self.parent_url if self.is_entity else self.url
@@ -274,3 +233,22 @@ class CrudComponent(HttpComponent):
 
     def delete_url(self, id_name: str) -> str:
         return f"{self.url}/{id_name}"
+
+    # callbacks
+
+    async def _head(self, response: ClientResponse) -> bool:
+        if response.status == 404:
+            return False
+        elif response.status == 200:
+            return True
+        else:  # pragma: no cover
+            raise MetablockResponseError(response)
+
+    async def _paginated(self, response: ClientResponse) -> Any:
+        next = response.links.get("next")
+        if isinstance(next, Mapping):
+            url = next.get("url")
+        else:
+            url = None
+        data = await self.cli.handle_response(response)
+        return (url, data)
