@@ -1,4 +1,4 @@
-from typing import List
+from typing import Any
 
 from .components import CrudComponent, MetablockEntity
 from .spaces import BlockPlugin
@@ -9,21 +9,17 @@ class Extension(MetablockEntity):
     """Object representing an Extension"""
 
 
-class Extensions(CrudComponent):
+class Extensions(CrudComponent[Extension]):
     """Extensions"""
 
-    Entity = Extension
-
 
 # Plugin
 class Plugin(MetablockEntity):
     """Object representing an Plugin"""
 
-    async def blocks(self) -> List[BlockPlugin]:
-        return await self.get(f"{self.url}/services")
+    async def blocks(self, **kwargs: Any) -> list[BlockPlugin]:
+        return await self.cli.get(f"{self.url}/services", **kwargs)
 
 
-class Plugins(CrudComponent):
+class Plugins(CrudComponent[Plugin]):
     """Plugins"""
-
-    Entity = Plugin
